package iXPaste::Model::Paste;

use strict;
use warnings;
use parent 'Catalyst::Model::File';
use Storable;
use Time::HiRes qw/gettimeofday/;

=head1 NAME

iXPaste::Model::Paste - Dateibasierter Storage fuer Paste-Eintraege

=head1 CONFIGURATION

Siehe auch L<Catalyst::Model::File>.

=cut

__PACKAGE__->config( root_dir => 'root/paste' );

=head1 DESCRIPTION

Dieses Model verwendet L<Storable> um Paste-Eintraege zu speichern und
zu laden.

=head1 METHODS

=head2 save( $params )

Speichert Paste-Eintrag im Dateisystem.

=cut

sub save {
    my ( $self, $params ) = @_;
    my $id = $self->_make_unique_id;
    $params->{id}    = $id;
    $params->{date}  = $self->_date_now();
    $params->{lines} = $self->_count_lines( $params->{content} );
    store $params, $self->file($id);
    return $id;
}

=head2 load( $id )

Laedt Paste-Eintrag vom Dateisystem.

=cut

sub load {
    my ( $self, $id ) = @_;
    my $file = $self->file($id);
    return undef unless defined $file and -r $file;
    return retrieve($file);
}

#
# Erzeugt eine eindeutige ID
#

sub _make_unique_id {
    my $self = shift;
    my %hash = ();
    my @chr  = ( 48 .. 57, 65 .. 90, 97 .. 122 );
    @hash{ 0 .. 60 } = @chr;
    my @seed = (localtime)[ 0 .. 5 ];
    $seed[ @seed - 1 ] -= 99;
    my $id = join( '', ( map { chr( $hash{$_} ) } @seed ), ( gettimeofday() )[1] );
    return $id;
}

#
# Zaehlt Zeilen des Paste-Eintrags
#

sub _count_lines {
    my ( $self, $content ) = @_;
    scalar @{ [ split /\n/, $content ] };
}

#
# Liefert das heutige Datum
#

sub _date_now {
    my ($self) = @_;
    my ( $sec, $min, $hour, $mday, $mon, $year ) = localtime(time);
    return sprintf '%4d-%02d-%02d %02d:%02d Uhr', $year + 1900, $mon + 1, $mday, $hour, $min;
}

=head1 AUTHOR

Johannes Plunien, Thomas Pietrzykowski

=head1 LICENSE

This library is free software. You can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

1;
